//	Zinc Application Framework - Z_APP.CPP
//	COPYRIGHT (C) 1990-1997.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

// The ZAF 5.0 Development Team:
//   Larry Macfarlane
//   Richard Paulsen
//   Mike Redd
//   Blake Jones
//   Michael Stone
//   James McPherson
//   David Frye
//   Brian Christiansen

#include <z_devall.hpp>
#include <z_win.hpp>
#include <z_app.hpp>
#include <z_i18n.hpp>
#include <z_store.hpp>
#include <z_error.hpp>
#include <z_help.hpp>

// ----- ZafApplication -----------------------------------------------------

ZafApplication::ZafApplication(int _argc, char **_argv) :
	argc(_argc), argv(ZAF_NULLP(ZafIChar *))
{
	// Allocate the display.
	zafDisplay = new ZafDisplay(_argc, _argv);

	// Initialize the data manager.
	zafDataManager = new ZafDataManager;

	// Call I18nAllocate() to reflect changes based on the display.
	ZafI18nData::I18nAllocate();

	// Allocate the arguments.
	if (argc)
	{
		argv = new ZafIChar *[argc];
		for (int i = 0; i < argc; i++)
			argv[i] = zafCodeSet->ConvertToZafString(_argv[i]);
	}

	// Call ResetI18n() to reflect changes based on the storage.
	zafSearchPath = new ZafPath(argv[0], true);
	zafI18nStorage = new ZafStorage(ZAF_ITEXT("i18n.znc"), ZAF_FILE_READ);
	if (zafI18nStorage->Error())
	{
		delete zafI18nStorage;
		zafI18nStorage = ZAF_NULLP(ZafFileSystem);
	}
	else
	{
		zafDataPersistence = new ZafDataPersistence(zafI18nStorage, 0);
		zafDataPersistence->AddDataConstructor(ZafLanguageData::className, ZafLanguageData::classID, &ZafLanguageData::Read);
		zafDataPersistence->AddDataConstructor(ZafLocaleData::className, ZafLocaleData::classID, &ZafLocaleData::Read);
		zafDataPersistence->AddDataConstructor(ZafCodeSetData::className, ZafCodeSetData::classID, &ZafCodeSetData::Read);
		ZafI18nData::ResetI18n();
	}

	// Allocate the event manager.
	zafEventManager = new ZafEventManager;
	zafEventManager->Add(new ZafKeyboard);
	zafEventManager->Add(new ZafMouse);
	zafEventManager->Add(new ZafCursor);
	zafEventManager->Add(new ZafHelpTips(D_ON, ZAF_HELPTIPS_BOTH));

	// Allocate the window manager.
	zafWindowManager = new ZafWindowManager;

	// Allocate the error system.
	zafErrorSystem = new ZafErrorSystem;
}

ZafApplication::~ZafApplication(void)
{
	// Free the arguments.
	if (argc)
	{
		for (int i = 0; i < argc; i++)
			delete []argv[i];
		delete []argv;
	}

	// Free the help and error systems.
	if (zafErrorSystem)
		delete zafErrorSystem;
	zafErrorSystem = ZAF_NULLP(ZafErrorStub);

	// Free the persistence modules.
	if (zafObjectPersistence)
		delete zafObjectPersistence;
	zafObjectPersistence = ZAF_NULLP(ZafObjectPersistence);
	if (zafDataPersistence)
		delete zafDataPersistence;
	zafDataPersistence = ZAF_NULLP(ZafDataPersistence);
	if (zafSearchPath)
		delete zafSearchPath;
	zafSearchPath = ZAF_NULLP(ZafPath);
	if (zafI18nStorage)
		delete zafI18nStorage;
	zafI18nStorage = ZAF_NULLP(ZafFileSystem);

	// Free the associated high-level managers.
	if (zafWindowManager)
		delete zafWindowManager;
	zafWindowManager = ZAF_NULLP(ZafWindowManager);
	if (zafEventManager)
		delete zafEventManager;
	zafEventManager = ZAF_NULLP(ZafEventManager);
	ZafI18nData::I18nFree(true);

	// Free the associated low-level managers.
	delete zafDataManager;
	zafDataManager = ZAF_NULLP(ZafDataManager);
	if (zafDisplay)
		delete zafDisplay;
	zafDisplay = ZAF_NULLP(ZafDisplay);
}

ZafEventType ZafApplication::Control(ZafQFlags flags)
{
	// Make sure there is a window attached to the window manager.
	if (!zafWindowManager->First())
		return (S_NO_OBJECT);

	// Wait for user response.
	ZafEventStruct event;
	ZafEventType ccode;
	do
	{
		if (zafEventManager->Get(event, flags) == 0)
			ccode = zafWindowManager->Event(event);
	} while (ccode != S_EXIT && ccode != S_NO_OBJECT);

	// Return the final code.
	return (ccode);
}

